@@ -34,6 +34,8 @@ RUN_TRAIN=#run_train#
 RUN_TEST=#run_test# # test will be run automatically, but not necessarily in this job
 IS_TEST=#is_test# # if True, start test in this job
 GRES_TEST=#gres_test#
+MEM_TEST=#mem_test#
+CPUS_TEST=#cpus_per_task_test#
 AUTO_RESUBMIT=#auto_resubmit# # resubmit the job if stopped due to time constraints
 RESUME_CHPT_PATH=#resume_chpt_path# # path to checkpoint file to resume training
 EXP_FILE_PATH=#exp_file_path#
@@ -210,11 +212,15 @@ if [ "$RUN_TEST" == True ]; then
         echo "Launching test job.."
         echo ------------------------------------------------------------
         GRES_TEST=$(awk -F= '/^GRES_TEST=/{print $2}' "$SUBMIT_FILE_PATH")
+        MEM_TEST=$(awk -F= '/^MEM_TEST=/{print $2}' "$SUBMIT_FILE_PATH")
+        CPUS_TEST=$(awk -F= '/^CPUS_TEST=/{print $2}' "$SUBMIT_FILE_PATH")
         sed -e "s|IS_TEST=False|IS_TEST=True|g" \
             -e "s|RUN_TRAIN=True|RUN_TRAIN=False|g" \
             -e "s|RUN_TEST=True|RUN_TEST=False|g" \
             -e "s|_train.|_test.|g" \
-            -e "s|^#SBATCH --gres==.*|#SBATCH --gres=$GRES_TEST|g" \
+            -e "s|^#SBATCH --gres=.*|#SBATCH --gres=$GRES_TEST|g" \
+            -e "s|^#SBATCH --mem=.*|#SBATCH --mem=$MEM_TEST|g" \
+            -e "s|^#SBATCH --cpus-per-task=.*|#SBATCH --cpus-per-task=$CPUS_TEST|g" \
             "$SCRATCH_SUBMIT_FILE_PATH" > "$SCRATCH_SUBMIT_FILE_PATH.resub"
         rm $SCRATCH_SUBMIT_FILE_PATH
         mv $SCRATCH_SUBMIT_FILE_PATH.resub $SCRATCH_SUBMIT_FILE_PATH
@@ -234,26 +240,76 @@ fi
 
 
 def recursive_dict_update(d_parent: dict, d_child: dict) -> dict:
-    """Recursively update the parent dictionary with values from the child dictionary.
+    """Merges two dictionaries recursively. The values of d_child will overwrite those in d_parent."""
+    result = copy.deepcopy(d_parent)
+
+    for key, value in d_child.items():
+        if isinstance(value, dict) and key in result and isinstance(result[key], dict):
+            result[key] = recursive_dict_update(result[key], value)
+        else:
+            result[key] = value
+
+    return result
+
+
+def load_json(path: str) -> dict:
+    """Load a JSON file and return its content as a dictionary."""
+    with open(path, encoding="utf8") as fp:
+        try:
+            data = json.load(fp)
+        except Exception as exc:
+            raise ValueError(
+                f"Error loading JSON file {path} (check syntax?)."
+            ) from exc
+
+    if data.get("globals") is None:
+        raise ValueError(
+            f"Error: experiment {path} does not contain 'globals' section. Check template!"
+        )
+    return data
+
+
+def get_parent_path(path: str, exp_file_path: str) -> str:
+    """Return the absolute path to the parent configuration file.
 
     Args:
-        d_parent (dict): The dictionary to be updated.
-        d_child (dict): The dictionary whose values will update the parent.
+        path (str): The path to the parent configuration file, which can be absolute or relative.
+        exp_file_path (str): The path to the current experiment file.
 
     Returns:
-        dict: A deep copy of the updated parent dictionary.
+        str: The absolute path to the parent configuration file.
     """
-    for key, value in d_child.items():
-        if (
-            isinstance(value, dict)
-            and key in d_parent
-            and isinstance(d_parent[key], dict)
-        ):
-            recursive_dict_update(d_parent[key], value)
-        else:
-            d_parent[key] = value
+    if path[0] == "/":
+        return path
+    return os.path.abspath(os.path.join(os.path.split(exp_file_path)[0], path))
 
-    return copy.deepcopy(d_parent)
+
+def load_conf_file(path) -> dict:
+    """Load an experiment configuration file, recursively merging parent configurations if specified.
+
+    Args:
+        path (str): Path to the experiment configuration JSON file.
+
+    Returns:
+        dict: The merged configuration as a DictToObj instance.
+    """
+    reached_leaf = False
+    conf = load_json(path)
+    parent_conf = conf.copy()
+    parent = conf.get("globals").get("parent", {})
+    conf["globals"]["parent_hierarchy"] = []
+
+    while not reached_leaf:
+        parent = parent_conf.get("globals").get("parent", {})
+        if parent == {}:
+            reached_leaf = True
+        else:
+            parent_path = get_parent_path(parent, path)
+            conf["globals"]["parent_hierarchy"].append(parent_path)
+            parent_conf = load_json(parent_path)
+            conf = recursive_dict_update(d_parent=parent_conf, d_child=conf)
+
+    return DictToObj(conf)
 
 
 class DictToObj:
@@ -308,53 +364,6 @@ class DictToObj:
         return res
 
 
-def parse_input_file(exp_file_path: str) -> DictToObj:
-    """Parse the experiment JSON file, handle parent inheritance, and return a DictToObj representation.
-
-    Args:
-        exp_file_path (str): Path to the experiment JSON file.
-
-    Returns:
-        DictToObj: Parsed experiment configuration as an object.
-    """
-    if not os.path.isfile(exp_file_path):
-        print(f"Error: The file '{exp_file_path}' does not exist.")
-        sys.exit(1)
-
-    try:
-        with open(exp_file_path, encoding="utf8") as file:
-            exp_file = json.load(file)
-    except json.JSONDecodeError as exc:
-        raise ValueError(
-            f"Error: The file '{exp_file_path}' is not a valid JSON file."
-        ) from exc
-
-    globals_def = exp_file.get("globals")
-    parent = globals_def.get("parent", {})
-    if parent != {}:
-        if parent[0] == "/":
-            parent_file_path = parent
-        else:
-            parent_file_path = os.path.abspath(
-                os.path.join(os.path.split(exp_file_path)[0], parent)
-            )
-
-        if not os.path.exists(parent_file_path):
-            raise FileNotFoundError(f"Error: File {parent_file_path} not found.")
-
-        with open(parent_file_path, encoding="utf8") as fp:
-            try:
-                parent_expfile = json.load(fp)
-            except Exception as exc:
-                raise ValueError(
-                    f"Error loading experiment file {parent_file_path} (check syntax?)."
-                ) from exc
-
-        exp_file = recursive_dict_update(d_parent=parent_expfile, d_child=exp_file)
-
-    return DictToObj(exp_file)
-
-
 def get_default_config(slurm_conf: Any) -> dict[str, Any]:
     """Return a job configuration dictionary with defaults, updated from the given SLURM config.
 
@@ -370,10 +379,12 @@ def get_default_config(slurm_conf: Any) -> dict[str, Any]:
         "job_time": None,
         "ntasks": 1,
         "cpus_per_task": 8,
+        "cpus_per_task_test": None,
         "nodes": 1,
         "gres": "gpu:1",
-        "gres_test": "gpu:1",
+        "gres_test": None,
         "mem": "32G",
+        "mem_test": None,
         "partition": None,
         "account": None,
         "run_train": True,
@@ -399,6 +410,11 @@ def get_default_config(slurm_conf: Any) -> dict[str, Any]:
         if val is not None:
             job_config[key] = val
 
+    # manually set test parameters if not set
+    for param in ["gres_test", "mem_test", "cpus_per_task_test"]:
+        if job_config[param] is None:
+            job_config[param] = job_config[param.replace("_test", "")]
+
     return job_config
 
 
@@ -467,8 +483,8 @@ def main() -> None:
             "Error: Exactly one argument is required which is the path to the JSON file."
         )
 
-    in_args: DictToObj = parse_input_file(sys.argv[1])
-    slurm_conf: Any = in_args.slurm_cluster
+    exp_config: DictToObj = load_conf_file(sys.argv[1])
+    slurm_conf: Any = exp_config.slurm_cluster
     if slurm_conf is None:
         raise ValueError(
             "Error: The 'slurm_cluster' section in the JSON config file is required to submit a job to the queue!"
@@ -485,7 +501,7 @@ def main() -> None:
 
     job_config["job_name"] = exp_name[:30].replace(" ", "_")
     job_config["user"] = getpass.getuser()
-    job_config["results_path"] = in_args.store.results_path
+    job_config["results_path"] = exp_config.store.results_path
     job_config["log_path"] = job_config["log_path"]
 
     # define path of submit script
